@@ -12,7 +12,7 @@ df <- function(n = 100) {
 mydf <- df()
 
 w <- c(rep(0, 50), rep(1, 50))
-mod1 <- glmboost(y ~ ., data = mydf, weights = w)
+mod1 <- glmboost(y ~ ., data = mydf, weights = w, center = FALSE)
 cf1 <- coef(mod1)
 
 ### hat matrix: fast for linear models
